@@ -336,15 +336,12 @@ static struct ofono_call *find_call_with_status(struct ofono_emulator *em,
 								int status)
 {
 	struct ofono_modem *modem = __ofono_atom_get_modem(em->atom);
-	struct ofono_atom *vc_atom;
 	struct ofono_voicecall *vc;
 
-	vc_atom = __ofono_modem_find_atom(modem, OFONO_ATOM_TYPE_VOICECALL);
-	if (vc_atom == NULL)
+	vc = __ofono_atom_find(OFONO_ATOM_TYPE_VOICECALL, modem);
+	if (vc == NULL)
 		return NULL;
 
-	vc = __ofono_atom_get_data(vc_atom);
-
 	return __ofono_voicecall_find_call_with_status(vc, status);
 }
 
@@ -424,13 +421,6 @@ static gboolean notify_ring(void *user_data)
 
 	c = find_call_with_status(em, CALL_STATUS_INCOMING);
 
-	/*
-	 * In case of waiting call becoming an incoming call, call status
-	 * change may not have been done yet, so try to find waiting call too
-	 */
-	if (c == NULL)
-		c = find_call_with_status(em, CALL_STATUS_WAITING);
-
 	if (c == NULL)
 		return TRUE;
 
@@ -1255,3 +1245,26 @@ start_ring:
 	em->callsetup_source = g_timeout_add_seconds(RING_TIMEOUT,
 							notify_ring, em);
 }
+
+void __ofono_emulator_set_indicator_forced(struct ofono_emulator *em,
+						const char *name, int value)
+{
+	int i;
+	struct indicator *ind;
+	char buf[20];
+
+	ind = find_indicator(em, name, &i);
+
+	if (ind == NULL || value < ind->min || value > ind->max)
+		return;
+
+	ind->value = value;
+
+	if (em->events_mode == 3 && em->events_ind && em->slc && ind->active) {
+		if (!g_at_server_command_pending(em->server)) {
+			sprintf(buf, "+CIEV: %d,%d", i, ind->value);
+			g_at_server_send_unsolicited(em->server, buf);
+		} else
+			ind->deferred = TRUE;
+	}
+}
