@@ -623,15 +623,18 @@ static DBusMessage *network_operator_register(DBusConnection *conn,
 	return NULL;
 }
 
-static GDBusMethodTable network_operator_methods[] = {
-	{ "GetProperties",  "",  "a{sv}",  network_operator_get_properties },
-	{ "Register",       "",  "",       network_operator_register,
-						G_DBUS_METHOD_FLAG_ASYNC },
+static const GDBusMethodTable network_operator_methods[] = {
+	{ GDBUS_METHOD("GetProperties",
+			NULL, GDBUS_ARGS({ "properties", "a{sv}" }),
+			network_operator_get_properties) },
+	{ GDBUS_ASYNC_METHOD("Register", NULL, NULL,
+						network_operator_register) },
 	{ }
 };
 
-static GDBusSignalTable network_operator_signals[] = {
-	{ "PropertyChanged",	"sv" },
+static const GDBusSignalTable network_operator_signals[] = {
+	{ GDBUS_SIGNAL("PropertyChanged",
+			GDBUS_ARGS({ "name", "s" }, { "value", "v" })) },
 	{ }
 };
 
@@ -1020,18 +1023,24 @@ static DBusMessage *network_get_operators(DBusConnection *conn,
 	return reply;
 }
 
-static GDBusMethodTable network_registration_methods[] = {
-	{ "GetProperties",  "",  "a{sv}",	network_get_properties },
-	{ "Register",       "",  "",		network_register,
-						G_DBUS_METHOD_FLAG_ASYNC },
-	{ "GetOperators",   "",  "a(oa{sv})",	network_get_operators },
-	{ "Scan",           "",  "a(oa{sv})",	network_scan,
-						G_DBUS_METHOD_FLAG_ASYNC },
+static const GDBusMethodTable network_registration_methods[] = {
+	{ GDBUS_METHOD("GetProperties",
+			NULL, GDBUS_ARGS({ "properties", "a{sv}" }),
+			network_get_properties) },
+	{ GDBUS_ASYNC_METHOD("Register",
+				NULL, NULL, network_register) },
+	{ GDBUS_METHOD("GetOperators",
+		NULL, GDBUS_ARGS({ "operators_with_properties", "a(oa{sv})" }),
+		network_get_operators) },
+	{ GDBUS_ASYNC_METHOD("Scan",
+		NULL, GDBUS_ARGS({ "operators_with_properties", "a(oa{sv})" }),
+		network_scan) },
 	{ }
 };
 
-static GDBusSignalTable network_registration_signals[] = {
-	{ "PropertyChanged",	"sv" },
+static const GDBusSignalTable network_registration_signals[] = {
+	{ GDBUS_SIGNAL("PropertyChanged",
+			GDBUS_ARGS({ "name", "s" }, { "value", "v" })) },
 	{ }
 };
 
